Package: ks
Version: 1.15.2
Date: 2026-05-09
Title: Kernel Smoothing
Authors@R: c(person("Tarn", "Duong", role=c("aut","cre"), email="tarn.duong@gmail.com", comment=c(ORCID="0000-0002-1198-3482")),
             person("Matt", "Wand", role="ctb", comment=c(ORCID="0000-0003-2555-896X")),
             person("Jose", "Chacon", role="ctb"),
             person("Artur", "Gramacki", role="ctb", comment=c(ORCID="0000-0002-1610-9743")))
Maintainer: Tarn Duong <tarn.duong@gmail.com>
Depends: R (>= 2.10.0)
Imports: FNN (>= 1.1), kernlab, KernSmooth (>= 2.22), Matrix, mclust,
        mgcv, multicool, mvtnorm (>= 1.0-0), pracma
Suggests: geometry, knitr, MASS, misc3d (>= 0.4-0), oz, plot3D, rgl (>=
        0.66), rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
Description: Kernel smoothers for univariate and multivariate data, with comprehensive visualisation and bandwidth selection capabilities, including for densities, density derivatives, cumulative distributions, clustering, classification, density ridges, significant modal regions, and two-sample hypothesis tests. Chacon & Duong (2018) <doi:10.1201/9780429485572>.   
License: GPL-2 | GPL-3
URL: https://mvstat.net/ks/, https://mvstat.net/mvksa/,
        https://github.com/cran/ks/
NeedsCompilation: yes
Packaged: 2026-05-09 12:34:47 UTC; tduong
Author: Tarn Duong [aut, cre] (ORCID: <https://orcid.org/0000-0002-1198-3482>),
  Matt Wand [ctb] (ORCID: <https://orcid.org/0000-0003-2555-896X>),
  Jose Chacon [ctb],
  Artur Gramacki [ctb] (ORCID: <https://orcid.org/0000-0002-1610-9743>)
Repository: CRAN
Date/Publication: 2026-05-09 14:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-09 16:55:55 UTC; unix
Archs: ks.so.dSYM
