importFrom("grDevices", chull, col2rgb, grey, hcl.colors, rgb)
importFrom("graphics", .filled.contour, box, contour, hist, image, legend, lines, pairs, par, persp, points, polygon, rect, rug)
importFrom("KernSmooth", dpik)
importFrom("Matrix", Diagonal, Matrix, norm)
importFrom("mclust", mclustBIC)
importFrom("mvtnorm", dmvnorm, pmvnorm, rmvnorm, dmvt, pmvt, rmvt)
importFrom("pracma", meshgrid, quiver)
importFrom("stats", IQR, cov, cutree, dbeta, dist, dnorm, ecdf, fivenum, fft, hclust, kmeans, loess, median, na.omit, nlm, optim, optimise, optimize, pchisq, pnorm, predict, qnorm, quantile, rbeta, rnorm, sd, smooth.spline, var)
importFrom("utils", head, setTxtProgressBar, tail, txtProgressBar)
    
useDynLib(ks, .registration=TRUE, .fixes="C_")

export(as.kde)
export(amise.mixt, ise.mixt, mise.mixt)
export(binning)
export(block.indices)
export(col.diverging)
export(compare, compare.kda.cv, compare.kda.diag.cv)
export(contourBreaks, .contourBreaks, contourLevels, contourSizes, contourProbs)
export(dkde, pkde, qkde, rkde)
export(dnorm.mixt, rnorm.mixt)
export(dmvnorm.mixt, rmvnorm.mixt)
export(dmvt.mixt, rmvt.mixt)
export(Hamise.mixt, Hamise.mixt.diag, hamise.mixt)
export(Hmise.mixt, Hmise.mixt.diag, hmise.mixt)
export(Hnm, Hnm.diag, hnm)
export(Hns, Hns.diag, hns, Hns.kcde, hns.kcde)
export(Hbcv, Hbcv.diag)
export(Hlscv, Hlscv.diag, hlscv, Hucv, Hucv.diag, hucv)
export(Hkda, Hkda.diag, hkda)
export(Hpi, Hpi.diag, hpi)
export(Hpi.kcde, Hpi.diag.kcde, hpi.kcde)
export(Hpi.kfe, Hpi.diag.kfe, hpi.kfe)
export(histde)
export(Hscv, Hscv.diag, hscv)
export(kcde)
export(kcopula, kcopula.de)
export(kcurv)
export(kda)
export(kdcde, dckde, reg.ucv)
export(kde)
export(kde.balloon)
export(kde.boundary)
export(kde.local.test)
export(kde.sp)
export(kde.test)
export(kde.truncate, kdde.truncate)
export(kdde)
export(kdr, kdr.segment)
export(kfe)
export(kfs)
export(kms, kms.part)
export(kroc)
export(ksupp, dwsupp)
export(Lpdiff)
export(matrix.sqrt)
export(mvnorm.mixt.mode, mvnorm.mixt.part)
export(now.time, now_time)
export(plotmixt)
export(pre.scale, pre.sphere)
export(rowKpow, getRow)
export(Sdr, Sdrv, mur, nur, nurs, Qr)
export(symconv.1d, symconv.nd)
export(vec, vech, invvec, invvech)

S3method(contourLevels, histde) 
S3method(contourLevels, kcde) 
S3method(contourLevels, kcopula.de) 
S3method(contourLevels, kda)
S3method(contourLevels, kde)
S3method(contourLevels, kdde)
S3method(contourLevels, kqde)
S3method(contourLevels, kqdde)
S3method(plot, histde)
S3method(plot, kcde)
S3method(plot, kcopula)
S3method(plot, kcopula.de)
S3method(plot, kda)
S3method(plot, kde)
S3method(plot, kde.loctest)
S3method(plot, kde.part)
S3method(plot, kdde)
S3method(plot, kdr)
S3method(plot, kfs)
S3method(plot, kms)
S3method(plot, kqde)
S3method(plot, kqdde)
S3method(plot, kroc)
S3method(plot, ksupp)
S3method(predict, histde)
S3method(predict, kda)
S3method(predict, kde)
S3method(predict, kcde)
S3method(predict, kcopula)
S3method(predict, kcopula.de)
S3method(predict, kdde)
S3method(predict, kfs)
S3method(predict, kqde)
S3method(predict, kqdde)
S3method(predict, kroc)
S3method(summary, kms)
S3method(summary, kroc)
