\name{ks-internal}
\alias{block.indices}
\alias{col.diverging}
\alias{dwsupp}
\alias{getRow}
\alias{kr}
\alias{Lpdiff}
\alias{matrix.sqrt}
\alias{mur}
\alias{now.time}
\alias{now_time}
\alias{nur}
\alias{nurs}
\alias{Qr}
\alias{rowKpow}
\alias{Sdr}
\alias{Sdrv}
\alias{symconv.1d}
\alias{symconv.nd}
\alias{reg.ucv}

\title{Internal functions in the ks library}

\description{
These functions are user-level but which the user is not required to use directly.
}

\value{The user is not required to use directly these outputs.}

\keyword{internal}
